@@ -24,6 +24,7 @@
 #include <Main.hpp>
 
 #include <Functions.hpp>
+#include <StreamMuxer.hpp>
 #include <SubsDec.hpp>
 #include <Demuxer.hpp>
 #include <Decoder.hpp>
@@ -227,6 +228,9 @@ void DemuxerThr::seek(bool doDemuxerSeek)
             playC.sPackets.unlock();
         }
 
+        if (doDemuxerSeek)
+            stopRecording();
+
         // Workaround: subtract 1 second for stepping backwards - sometimes FFmpeg doesn't seek to
         // key frame (why?) or PTS is higher than DTS. This also doesn't resolve all rare issues.
         if (doDemuxerSeek && demuxer->seek(playC.seekTo - (stepBackwards ? 1.0 : 0.0), backward))
@@ -339,8 +343,22 @@ void DemuxerThr::checkReadyWrite(AVThread *avThr)
     }
 }
 
+void DemuxerThr::startRecording()
+{
+    if (isDemuxerReady())
+        m_recording = true;
+    else if (!m_recording)
+        emit recording(false, true);
+}
+void DemuxerThr::stopRecording()
+{
+    m_recording = false;
+}
+
 void DemuxerThr::run()
 {
+    const auto origUrl = url;
+
     emit playC.chText(tr("Opening"));
     emit playC.setCurrentPlaying();
 
@@ -350,6 +368,14 @@ void DemuxerThr::run()
     if (ioCtrl.isAborted())
         return end();
 
+    if (Functions::isResourcePlaylist(url))
+    {
+        end();
+        emit QMPlay2Core.processParam("remove", origUrl);
+        emit QMPlay2Core.processParam("open", url);
+        return;
+    }
+
     if (!Demuxer::create(url, demuxer) || demuxer.isAborted())
     {
         if (!demuxer.isAborted() && !demuxer)
@@ -503,6 +529,11 @@ void DemuxerThr::run()
     bool firstWaitForData = true;
     bool canWaitForData = true;
 
+    QHash<int, int> recStreamsMap;
+
+    if (unknownLength)
+        emit allowRecording(true);
+
     while (!demuxer.isAborted())
     {
         {
@@ -512,6 +543,11 @@ void DemuxerThr::run()
                 break; //Repeat seek failed - break.
         }
 
+        if (m_recording && !m_recMuxer)
+            startRecordingInternal(recStreamsMap);
+        else if (!m_recording && m_recMuxer)
+            stopRecordingInternal(recStreamsMap);
+
         AVThread *aThr = (AVThread *)playC.aThr, *vThr = (AVThread *)playC.vThr;
 
         checkReadyWrite(aThr);
@@ -668,6 +704,12 @@ void DemuxerThr::run()
             if (streamIdx < 0 || playC.seekTo == SEEK_STREAM_RELOAD)
                 continue;
 
+            if (int recStreamIdx = recStreamsMap.value(streamIdx, -1); recStreamIdx > -1)
+            {
+                Q_ASSERT(m_recMuxer);
+                m_recMuxer->write(packet, recStreamIdx);
+            }
+
             if (streamIdx == playC.audioStream)
                 playC.aPackets.put(packet);
             else if (streamIdx == playC.videoStream)
@@ -702,12 +744,71 @@ void DemuxerThr::run()
         }
     }
 
+    if (m_recMuxer)
+        stopRecordingInternal(recStreamsMap);
+
+    emit allowRecording(false);
+
     emit QMPlay2Core.updatePlaying(false, title, artist, album, round(demuxer->length()), false, updatePlayingName);
 
     playC.endOfStream = playC.canUpdatePos = false; //to musi tu być!
     end();
 }
 
+void DemuxerThr::startRecordingInternal(QHash<int, int> &recStreamsMap)
+{
+    QList<StreamInfo *> recStreamsInfo;
+    Q_ASSERT(recStreamsMap.isEmpty());
+    Q_ASSERT(m_recording);
+    Q_ASSERT(!m_recMuxer);
+    auto pushStream = [&](int streamIdx) {
+        if (streamIdx > -1)
+        {
+            recStreamsMap[streamIdx] = recStreamsInfo.size();
+            recStreamsInfo.push_back(demuxer->streamsInfo().at(streamIdx));
+        }
+    };
+    pushStream(playC.videoStream);
+    pushStream(playC.audioStream);
+    pushStream(playC.subtitlesStream);
+
+    const auto [ext, fmt] = Functions::determineExtFmt(recStreamsInfo);
+
+    const auto dir = QMPlay2Core.getSettings().getString("OutputFilePath");
+    const auto fileName = Functions::getSeqFile(dir, "." + ext, "rec");
+    m_recMuxer = std::make_unique<StreamMuxer>(dir + "/" + fileName, recStreamsInfo, fmt, true);
+    if (m_recMuxer->isOk())
+    {
+        emit recording(true, false, fileName);
+        auto writePackets = [&](PacketBuffer &packets, int packetsStreamIdx) {
+            if (int recStreamIdx = recStreamsMap.value(packetsStreamIdx, -1); recStreamIdx > -1)
+            {
+                packets.iterate([&](const Packet &packet) {
+                    m_recMuxer->write(packet, recStreamIdx);
+                });
+            }
+        };
+        writePackets(playC.vPackets, playC.videoStream);
+        writePackets(playC.aPackets, playC.audioStream);
+        writePackets(playC.sPackets, playC.subtitlesStream);
+    }
+    else
+    {
+        m_recording = false;
+        m_recMuxer.reset();
+        recStreamsMap.clear();
+        emit recording(false, true);
+    }
+}
+void DemuxerThr::stopRecordingInternal(QHash<int, int> &recStreamsMap)
+{
+    Q_ASSERT(m_recMuxer);
+    m_recording = false;
+    m_recMuxer.reset();
+    recStreamsMap.clear();
+    emit recording(false, false);
+}
+
 inline void DemuxerThr::ensureTrueUpdateBuffered()
 {
     time = Functions::gettime() - updateBufferedTime; //zapewni, że updateBuffered będzie na "true";
@@ -1154,7 +1255,7 @@ void DemuxerThr::emitInfo()
         ++i;
     }
     i = 0;
-    for (const QString &fName : qAsConst(playC.fileSubsList))
+    for (const QString &fName : std::as_const(playC.fileSubsList))
         addSubtitleStream(fName == playC.fileSubs, subtitlesStreams, i++, ++subtitlesStreamCount, "fileSubs", QString(), Functions::fileName(fName), subtitlesStreamsMenu);
 
     if (!videoStreams.isEmpty())
