@@ -114,6 +114,16 @@ void PlayClass::play(const QString &_url)
             demuxThr->updateBufferedSeconds = QMPlay2Core.getSettings().getBool("ShowBufferedTimeOnSlider");
 
             connect(demuxThr, SIGNAL(load(Demuxer *)), this, SLOT(load(Demuxer *)));
+            connect(demuxThr, &DemuxerThr::allowRecording, this, &PlayClass::allowRecording);
+            connect(demuxThr, &DemuxerThr::recording, this, [this](bool status, bool error, const QString &fileName) {
+                if (error)
+                    messageAndOSD(tr("Recording error"));
+                else if (status)
+                    messageAndOSD(tr("Recording started: %1").arg(fileName));
+                else
+                    messageAndOSD(tr("Recording stopped"));
+                emit recording(status && !error);
+            });
             connect(demuxThr, SIGNAL(finished()), this, SLOT(demuxThrFinished()));
 
             if (!QMPlay2Core.getSettings().getBool("KeepZoom"))
@@ -275,14 +285,14 @@ void PlayClass::seek(double pos, bool allowAccurate)
 void PlayClass::chStream(const QString &s)
 {
     if (s.startsWith("audio"))
-        chosenAudioStream = QStringView(s).right(s.length() - 5)MAYBE_TO_STRING.toInt();
+        chosenAudioStream = QStringView(s).right(s.length() - 5).toInt();
     else if (s.startsWith("video"))
-        chosenVideoStream = QStringView(s).right(s.length() - 5)MAYBE_TO_STRING.toInt();
+        chosenVideoStream = QStringView(s).right(s.length() - 5).toInt();
     else if (s.startsWith("subtitles"))
-        chosenSubtitlesStream = QStringView(s).right(s.length() - 9)MAYBE_TO_STRING.toInt();
+        chosenSubtitlesStream = QStringView(s).right(s.length() - 9).toInt();
     else if (s.startsWith("fileSubs"))
     {
-        int idx = QStringView(s).right(s.length() - 8)MAYBE_TO_STRING.toInt();
+        int idx = QStringView(s).right(s.length() - 8).toInt();
         if (fileSubsList.count() > idx)
             loadSubsFile(fileSubsList[idx]);
     }
@@ -435,6 +445,20 @@ void PlayClass::setKeepAudioPitch(bool keep)
     QMPlay2Core.getSettings().set("KeepAudioPitch", keepAudioPitch);
 }
 
+void PlayClass::setRecording(bool checked)
+{
+    if (!demuxThr)
+    {
+        emit recording(false);
+        return;
+    }
+
+    if (checked)
+        demuxThr->startRecording();
+    else
+        demuxThr->stopRecording();
+}
+
 inline bool PlayClass::hasVideoStream()
 {
     return vThr && demuxThr && demuxThr->demuxer && videoStream > -1;
@@ -1269,16 +1293,49 @@ static Decoder *loadStream(
     }
     else
     {
+        const int desiredVideoHeight = (type == AVMEDIA_TYPE_VIDEO)
+            ? QMPlay2Core.getSettings().getInt("DesiredVideoHeight") * 6 / 5
+            : 0
+        ;
         int defaultStream = -1, chosenLangStream = -1;
+        int minVideoHeight = 0, minVideoHeightStream = -1, desiredVideoHeightStream = -1;
         for (int i = 0; i < streams.count(); ++i)
         {
-            if (streams[i]->params->codec_type == type)
+            const auto params = streams[i]->params;
+            if (params->codec_type == type)
             {
                 if (defaultStream < 0 && streams[i]->is_default)
                     defaultStream = i;
+                if (desiredVideoHeight > 0)
+                {
+                    // Find desired video height with highest bitrate
+                    if (params->height <= desiredVideoHeight)
+                    {
+                        if (desiredVideoHeightStream == -1)
+                        {
+                            desiredVideoHeightStream = i;
+                        }
+                        else
+                        {
+                            const auto desiredStreamParams = streams[desiredVideoHeightStream]->params;
+                            if (desiredStreamParams->height <= params->height
+                                    && (desiredStreamParams->height != params->height
+                                        || params->bit_rate <= 0
+                                        || params->bit_rate > desiredStreamParams->bit_rate))
+                            {
+                                desiredVideoHeightStream = i;
+                            }
+                        }
+                    }
+                    if (minVideoHeight == 0 || params->height < minVideoHeight)
+                    {
+                        minVideoHeight = params->height;
+                        minVideoHeightStream = i;
+                    }
+                }
                 if (!lang.isEmpty() && chosenLangStream < 0)
                 {
-                    for (const QMPlay2Tag &tag : qAsConst(streams[i]->other_info))
+                    for (const QMPlay2Tag &tag : std::as_const(streams[i]->other_info))
                     {
                         if (tag.first.toInt() == QMPLAY2_TAG_LANGUAGE)
                         {
@@ -1290,6 +1347,10 @@ static Decoder *loadStream(
                 }
             }
         }
+        if (desiredVideoHeightStream != -1)
+            defaultStream = desiredVideoHeightStream;
+        else if (minVideoHeightStream > -1)
+            defaultStream = minVideoHeightStream;
         if (chosenLangStream > -1)
             defaultStream = chosenLangStream;
         for (int i = 0; i < streams.count(); ++i)
