@@ -101,14 +101,6 @@ private:
 
 class DownloadListW final : public QTreeWidget
 {
-    friend class Downloader;
-public:
-    inline QString getDownloadsDirPath()
-    {
-        return downloadsDirPath;
-    }
-private:
-    QString downloadsDirPath;
 };
 
 /**/
@@ -165,7 +157,6 @@ private:
     bool modifyConvertAction(QAction *action, bool addRemoveButton = true);
 
 private slots:
-    void setDownloadsDir();
     void clearFinished();
     void addUrl();
     void download();
@@ -178,7 +169,7 @@ private:
 
     QGridLayout *layout;
     DownloadListW *downloadLW;
-    QToolButton *setDownloadsDirB, *clearFinishedB, *addUrlB;
+    QToolButton *clearFinishedB, *addUrlB;
 
     QToolButton *m_convertsPresetsB;
     QMenu *m_convertsMenu;
